async def merge_attendance(attendance_rows, extra_attendance_rows):
    present = attendance_rows["present"]
    absent = attendance_rows["absent"]

    for date, sessions in extra_attendance_rows["sessions"].items():
        session_on_day = attendance_rows["sessions"].get(date, sessions)
        if session_on_day == sessions:
            present += sessions["present"]
            absent += sessions["absent"]
            for period in sessions["classes"]:
                subject = attendance_rows["subjects"].get(period["subject"], {"present": 0, "absent": 0})
                subject["present"] += period["did_attend"]
                subject["absent"] -= not period["did_attend"]
                attendance_rows["subjects"][period["subject"]] = subject
        else:
            for period in sessions["classes"]:
                merged = False
                for period_on_day in session_on_day["classes"]:
                    if (
                        period["subject"] == period_on_day["subject"]
                        and period["start"] == period_on_day["start"]
                        and period["end"] == period_on_day["end"]
                    ):
                        if period["did_attend"] != period_on_day["did_attend"] and period_on_day["did_attend"] is False:
                            present += 1
                            absent -= 1
                            period_on_day["did_attend"] = True
                            subject = attendance_rows["subjects"].get(period["subject"], {"present": 0, "absent": 0})
                            subject["present"] += 1
                            subject["absent"] -= 1
                            attendance_rows["subjects"][period["subject"]] = subject

                        merged = True

                if not merged:
                    present += period["did_attend"]
                    absent += not period["did_attend"]
                    subject = attendance_rows["subjects"].get(period["subject"], {"present": 0, "absent": 0})
                    subject["present"] += period["did_attend"]
                    subject["absent"] -= not period["did_attend"]
                    attendance_rows["subjects"][period["subject"]] = subject
                    session_on_day["classes"].append(period)

        attendance_rows["sessions"][date] = session_on_day

    total = present + absent
    percent = round((present / total) * 100, 2)

    total_attendance_stats = {
        "present": present,
        "absent": absent,
        "total": total,
        "percent": percent,
        "subjects": attendance_rows["subjects"],
        "sessions": attendance_rows["sessions"],
    }
    return total_attendance_stats



import re
from datetime import datetime

from vasvscrapper.enums import NewsPatternsEnum


async def checker(obj):
    ret_obj = obj
    count = 0

    categories = []

    description = obj["content"].lower()  # noqa: F841
    for news_pattern in NewsPatternsEnum.values():
        pattern, key = news_pattern["pattern"], news_pattern["key"]

        condition = re.sub(r"(?P<name>[a-z]+)", r"'\1'", pattern)
        condition = re.sub(r"&", f" in description and ", condition)
        condition = re.sub(r"\|", f" in description or ", condition)
        condition = re.sub(r"\) in description", f")", condition)
        condition = re.sub(r"(?P<name>[a-z])\'\)", fr"\1' in description)", condition)
        condition = re.sub(r"\$REMAINING", "count == 0", condition)

        if eval(condition):
            categories.append(key)
            count += 1

    ret_obj["categories"] = str(categories)
    return ret_obj


async def news_scrapper(news_page_soup):
    news_table = news_page_soup.find(id="tblGrid")
    news_block = news_table("tbody")[0]("tr")
    news_objs = []
    for news_row in news_block:
        news_obj = {}
        news_date = datetime.strptime(news_row.find(class_="date").text, "%d-%b-%Y").strftime("%Y-%m-%d")

        news_link = news_row.find("a").attrs.get("href")
        news_category = news_row.find("a").find("em").text
        news_content = (
            news_row.find("a")
            .text[len(news_category) + 1 :]
            .replace("\u201c", "'")
            .replace("\u201d", "'")
            .replace('"', "'")
            .replace("\u2018", "'")
            .replace("\u2019", "'")
            .replace("\u2013", "-")
        )

        news_obj["date"] = news_date
        news_obj["link"] = news_link
        news_obj["content"] = news_content
        news_obj["index"] = news_category[1:-1]

        news_objs.append(await checker(news_obj))

    return news_objs



"""
Attendance scrapper which extract information from an attendance
page and return information in dictionary.
"""

from vasvscrapper.attendance.get_attendance import get_attendance

__all__ = ["get_attendance"]



"""
Login Status codes representing failures and success states
in scrapping a student details.
"""

LOGIN_CREDENTIALS_INVALID = "LOGIN__CREDENTIALS_INVALID"
LOGIN_VCE_CRASHED = "LOGIN__VCE_CRASHED"
LOGIN_SUCCESS = "LOGIN__SUCCESS"
LOGIN_SCRAPPER_FAILED = "LOGIN__SCRAPPER_FAILED"



"""
    Makes a network request and returns status
    and BeautifulSoup object of the request.
    """
from .async_requests import get_attendance_html, get_login_html, get_news_html, get_results_html

__all__ = ["get_login_html", "get_news_html", "get_attendance_html", "get_results_html"]



"""
News scrapper which extract information from news page
and return information in dictionary.
"""

from .news_scrapper import get_news, get_news_in_range

__all__ = ["get_news", "get_news_in_range"]



from enum import Enum

from yarl import URL


class ConstEnum(Enum):
    @classmethod
    def keys(cls):
        return list(map(lambda x: x.name, cls))

    @classmethod
    def as_dict(cls):
        key_val = {}
        for item in cls:
            key_val[item.name] = item.value

        return key_val

    @classmethod
    def values(cls):
        return list(map(lambda x: x.value, cls))


class BranchesEnum(ConstEnum):
    """
    Enum representing various Branches in VCE.
    """

    CIVIL = {"name": "Civil Engineering", "branch_code": "732"}
    CSE = {"name": "Computer Science and Engineering", "branch_code": "733"}
    EEE = {"name": "Electrical and Electronics Engineering", "branch_code": "734"}
    ECE = {"name": "Electronics and Communication Engineering", "branch_code": "735"}
    MECH = {"name": "Mechanical Engineering", "branch_code": "736"}
    IT = {"name": "Information Technology", "branch_code": "737"}

    @classmethod
    def branch_name_map(cls):
        branches = {}
        for branch in cls.values():
            branches[branch["branch_code"]] = branch["name"]

        return branches


class NewsPatternsEnum(ConstEnum):
    """
    News patterns to categorize news.
    pattern repr

    - Outer parenthesis is important ()
    - If you want "time" and "table" both words to be in a news then we represent it as "time&table"
    - If you want "suppl" and "backlog" either of the word to be in news, we represent it as "suppl|backlog"
    - Group the works using parenthesis "((suppl|backlog)&(time&table))" to represent "Supplementary TimeTable" category
    """

    SUPPLEMENTARY = {"pattern": "(suppl|backlog|makeup)", "key": "Supplementary"}
    EXAMINATION = {"pattern": "(exam|test|internal)", "key": "Examination"}
    TIMETABLE = {"pattern": "(time&table)", "key": "Timetable"}
    REVALUATION = {"pattern": "(revaluation)", "key": "Revaluation"}
    RESULT = {"pattern": "(result)", "key": "Result"}
    ADMISSION = {"pattern": "(admission)", "key": "Admission"}
    FEE = {"pattern": "(fee)", "key": "Fee"}
    OPEN_ELECTIVE = {"pattern": "(open&elective)", "key": "Open Elective"}
    INFORMATION = {"pattern": "$REMAINING", "key": "Information"}

    @classmethod
    def get_news_keys(cls):
        return list(map(lambda x: x.value["key"], cls))


class LoginUrlEnum(ConstEnum):
    """
    Enum representing success and failure urls in VCE.
    """

    SUCCESS = URL("https://vce.ac.in/student_info.aspx")
    FAIL = URL("https://vce.ac.in/")



"""
Some of the enums used in scrappers for performing
various operations in extraction.
"""

from .scrapper_enums import BranchesEnum, LoginUrlEnum, NewsPatternsEnum

__all__ = ["BranchesEnum", "NewsPatternsEnum", "LoginUrlEnum"]



from vasvscrapper.enums.scrapper_enums import LoginUrlEnum
from vasvscrapper.login.utilities import login_scrapper
from vasvscrapper.network_requests import get_login_html
from vasvscrapper.status_codes.login import (
    LOGIN_CREDENTIALS_INVALID,
    LOGIN_SCRAPPER_FAILED,
    LOGIN_SUCCESS,
    LOGIN_VCE_CRASHED,
)


async def login_student(roll_number, password) -> dict:
    """
    Extracts the student information of a student
    and return it in a dictionary.

    PARAMETERS
    ----------
    roll_number    : str
        Roll number of the student in vce.
    password       : str
        Password of the student in vce portal.

    RETURNS
    -------
    obj : dict
        Extracted information of a given student.
    """

    user = {}
    try:
        status_code, profile_page_soup, url = await get_login_html(roll_number, password)

        if url == LoginUrlEnum.FAIL.value:
            user["status"] = LOGIN_CREDENTIALS_INVALID

        elif len(profile_page_soup("span")) == 0 or status_code > 400:
            user["status"] = LOGIN_VCE_CRASHED

        else:
            user = await login_scrapper(profile_page_soup)
            user["status"] = LOGIN_SUCCESS

        return user
    except TimeoutError:
        user["status"] = LOGIN_VCE_CRASHED

    except Exception:
        user["status"] = LOGIN_SCRAPPER_FAILED



import datetime

from vasvscrapper.attendance.utilities.curate_attendance import curate_attendance


async def get_timestamps(date_row):
    timestamps = {}
    timestamps_rows = date_row("tr")[0]
    timestamps_count = 0
    for i in timestamps_rows("td")[3:-1]:
        timestamps_count += 1
        timestamps[f"time_{timestamps_count}"] = [k.strip() for k in i.text.split("-")]
    return timestamps, timestamps_count


async def attendance_scrapper(attendance_table, current=False, extra=False):
    row_split = 2
    col_split = -3

    if extra:
        row_split = 1
        col_split = -1

    stats = {
        "present": 0,
        "absent": 0,
        "subjects": {},
        "sessions": {},
    }
    if attendance_table is not None and (current or extra):
        classes_rows = attendance_table("tr")[row_split:]
        subjects = {}

        present = 0
        absent = 0

        timestamps, timestamps_count = await get_timestamps(attendance_table)

        attendance_rows = {}
        for i in classes_rows:

            if int(i("td")[-1].text) != 0:
                current_date = datetime.datetime.strptime(i("td")[2].text, "%d-%b-%Y").date()
                attendance_row = {"present": 0, "absent": 0, "total": 0, "classes": []}
                attendance = {}

                k = 0
                for j in i("td")[3:col_split]:
                    colspan = int(j.attrs.get("colspan", 1))
                    if colspan in [1, 2, 3, 4, 5, 6]:
                        for _ in range(colspan):
                            k += 1
                            attendance[f"class_{k}"] = j.text.strip().split("(")[0]
                            if j.attrs.get("bgcolor") == "Dodgerblue" or j.attrs.get("bgcolor") == "Green":
                                attendance[f"status_{k}"] = False
                            else:
                                attendance[f"status_{k}"] = True

                attendance_row = await curate_attendance(
                    attendance, attendance_row, k, subjects, timestamps_count, timestamps
                )
                present += attendance_row["present"]
                absent += attendance_row["absent"]
                attendance_rows[current_date] = attendance_row

        total = present + absent
        if total > 0:
            stats = {
                "present": present,
                "absent": absent,
                "subjects": subjects,
                "sessions": attendance_rows,
            }

    return stats



from datetime import datetime

from vasvscrapper.enums import BranchesEnum

labels_map = {
    "Hall Ticket No.": "roll_number",
    "Name": "name",
    "Date of Birth": "dob",
    "Father Name": "fathers_name",
    "Admission Date": "admission_date",
    "Branch": "branch",
    "Year": "current_year",
    "Status": "current_status",
    "Semester": "current_sem",
    "Section": "section",
}


def curate_values(result: dict):
    branches = BranchesEnum.branch_name_map()

    for label, value in result.items():
        if label in ["current_year", "current_sem"]:
            result[label] = int(value)

        elif label in ["name"]:
            result[label] = value.title()

        elif label == "branch":
            result[label] = branches.get(result["roll_number"][8:11], result["branch"])

        elif value == "-":
            result[label] = ""

    return result


def get_values_by_labels(student_obj):
    result = dict.fromkeys(labels_map.values(), "")

    for obj in student_obj:
        label = labels_map[obj("td")[0].text.strip()]
        value = obj("td")[1].text.strip()

        result[label] = value

    return curate_values(result)


async def login_scrapper(profile_page_soup):
    semesters = []
    student_details_block = profile_page_soup.find(id="divStudInfo")
    student_details_links = profile_page_soup.find(id="divAttSummary")

    student_info = student_details_block("tr")[1:]
    attendance_links_rows = student_details_links("tr")

    user = get_values_by_labels(student_info)

    for i in attendance_links_rows[1:]:
        semesters.append(
            {
                "sem": int(i("td")[1].text),
                "year": i("td")[2].text,
                "status": i("td")[3].text,
                "start_date": datetime.strptime(i("td")[4].text, "%d-%b-%Y").strftime("%Y-%m-%d"),
                "end_date": datetime.strptime(i("td")[5].text, "%d-%b-%Y").strftime("%Y-%m-%d"),
                "attendance_link": i("td")[-2]("a")[0].attrs.get("onclick", "href")[7:-2],
                "result_link": i("td")[-1]("a")[0].attrs.get("onclick", "href")[7:-2],
            }
        )

    user["semesters"] = semesters
    return user



"""
Attendance status codes representing failures and success
states in scrapping attendance.
"""

ATTENDANCE_VCE_CRASHED = "ATTENDANCE__VCE_CRASHED"
ATTENDANCE_SCRAPPER_FAILED = "ATTENDANCE__SCRAPPER_FAILED"
ATTENDANCE_SUCCESS = "ATTENDANCE__SUCCESS"



from asyncio.exceptions import TimeoutError

from vasvscrapper.attendance.utilities import attendance_scrapper, merge_attendance
from vasvscrapper.network_requests import get_attendance_html
from vasvscrapper.status_codes.attendance import ATTENDANCE_SCRAPPER_FAILED, ATTENDANCE_SUCCESS, ATTENDANCE_VCE_CRASHED


async def get_attendance(link, current=True, extra=True):
    """
    Extracts the attendance information and return a dictionary.

    PARAMETERS
    ----------
    link    : str
        Link of the result page
    current : bool
        If the attendance has to include current running attendance (default if True).
    extra   : bool
        If the attendance has to include extra attendance (default is True).

    RETURNS
    -------
    obj : dict
        Extracted information for a given page
    """

    attendance = {}
    try:
        status, attendance_details_soup = await get_attendance_html(link, 120)

        if status >= 400:
            attendance["status"] = ATTENDANCE_VCE_CRASHED

        else:
            attendance_table = attendance_details_soup.find(id="TdDispAttInfo")
            extra_attendance_table = attendance_details_soup.find(id="TdDispAdditionalAttInfo")

            attendance_stats = await attendance_scrapper(attendance_table, current)
            extra_attendance_stats = await attendance_scrapper(extra_attendance_table, extra)

            attendance = await merge_attendance(attendance_stats, extra_attendance_stats)
            attendance["status"] = ATTENDANCE_SUCCESS

        return attendance

    except TimeoutError:
        attendance["status"] = ATTENDANCE_VCE_CRASHED

    except Exception:
        attendance["status"] = ATTENDANCE_SCRAPPER_FAILED



import asyncio

from vasvscrapper.network_requests import get_news_html
from vasvscrapper.news.utilities import news_scrapper
from vasvscrapper.status_codes.news import NEWS_SCRAPPER_FAILED, NEWS_SUCCESS, NEWS_VCE_CRASHED


async def get_news(page_number):
    """
    Extracts the news information of a given page
    number and return it in a dictionary.

    PARAMETERS
    ----------
    page_number    : int
        Page number in vce website

    RETURNS
    -------
    obj : dict
        Extracted information for a given page
    """

    news = {"page_number": page_number}
    try:
        status, news_page_soup = await get_news_html(page_number)

        if status >= 400:
            news["status"] = NEWS_VCE_CRASHED

        else:
            news_objs = await news_scrapper(news_page_soup)

            news["news"] = news_objs
            news["status"] = NEWS_SUCCESS

        return news

    except Exception:
        news["status"] = NEWS_SCRAPPER_FAILED
        return news


async def get_news_in_range(start_page, end_page, limit=25) -> dict:
    """
    Extracts the news information of a given page numbers
    range and return it in a dictionary.

    PARAMETERS
    ----------
    start_page    : int
        Starting page number of vce website
    end_page      : int
        Ending page number of vce website
    limit         : int, optional
        Limit the ending page to certain page (default is 25)

    RETURNS
    -------
    obj : dict
        Extracted information for a all the pages in given range
    """

    if end_page > limit:
        end_page = limit
    tasks = []
    for page_number in range(start_page, end_page + 1):
        tasks.append(asyncio.create_task(get_news(page_number)))

    all_news = await asyncio.gather(*tasks)

    result_news_objs = {
        "news": [],
        "status": {
            NEWS_SUCCESS: [],
            NEWS_VCE_CRASHED: [],
            NEWS_SCRAPPER_FAILED: [],
        },
    }

    for news in all_news:
        status = news["status"]
        if news["status"] in (NEWS_VCE_CRASHED, NEWS_SCRAPPER_FAILED):
            result_news_objs["status"][status].append(news["page_number"])

        else:
            result_news_objs["status"][status].append(news["page_number"])
            result_news_objs["news"].extend(news["news"])

    return result_news_objs



async def curate_attendance(attendance, attendance_row, session_row_count, subjects, timestamps_count, timestamps):
    day_present = 0
    day_absent = 0

    #  - - in series then remove one - and shift left
    if session_row_count > timestamps_count:
        for obj_index in range(1, session_row_count):
            if attendance[f"class_{obj_index}"] == attendance[f"class_{obj_index + 1}"] == "-":
                for new_index in range(obj_index, session_row_count):
                    attendance[f"class_{new_index}"] = attendance[f"class_{new_index + 1}"]
                    attendance[f"status_{new_index}"] = attendance[f"status_{new_index + 1}"]

                del attendance[f"class_{session_row_count}"]
                del attendance[f"status_{session_row_count}"]
                session_row_count -= 1
                if session_row_count == timestamps_count:
                    break

    # only - is available then shift left
    rem = 0
    if session_row_count > timestamps_count:
        for obj_index in range(1, session_row_count + 1):
            if attendance[f"class_{obj_index - rem}"] == "-":
                for new_index in range(obj_index - rem, session_row_count - rem):
                    attendance[f"class_{new_index}"] = attendance[f"class_{new_index + 1}"]
                    attendance[f"status_{new_index}"] = attendance[f"status_{new_index + 1}"]
                del attendance[f"class_{session_row_count - rem}"]
                del attendance[f"status_{session_row_count - rem}"]
                rem += 1
                if session_row_count - rem == timestamps_count:
                    break

    for obj_index in range(1, session_row_count - 1):
        if (
            float(timestamps[f"time_{obj_index}"][1]) > float(timestamps[f"time_{obj_index + 1}"][0])
            and attendance[f"class_{obj_index}"] != "-"
            and attendance[f"class_{obj_index + 1}"] != "-"
        ):
            req = -1
            for i in range(obj_index + 1, session_row_count):
                if attendance[f"class_{i}"] == "-":
                    req = i
                    break

            for j in range(req, obj_index, -1):
                attendance[f"class_{j + 1}"], attendance[f"class_{j}"] = (
                    attendance[f"class_{j}"],
                    attendance[f"class_{j + 1}"],
                )
                attendance[f"status_{j + 1}"], attendance[f"status_{j}"] = (
                    attendance[f"status_{j}"],
                    attendance[f"status_{j + 1}"],
                )

    sessions = []
    for obj_index in range(1, session_row_count + 1):
        if attendance[f"class_{obj_index}"] != "-":
            end = f'{float(timestamps[f"time_{obj_index}"][1]) + 0.1:.2f}'
            if end[-2:] == "60":
                end = f"{float(end) + 0.4:.2f}"

            obj = {
                "subject": attendance[f"class_{obj_index}"],
                "did_attend": attendance[f"status_{obj_index}"],
                "start": timestamps[f"time_{obj_index}"][0],
                "end": end,
            }
            subjects[obj["subject"]] = subjects.get(obj["subject"], {"present": 0, "absent": 0})
            subjects[obj["subject"]]["present"] = subjects[obj["subject"]]["present"] + int(obj["did_attend"])
            subjects[obj["subject"]]["absent"] = subjects[obj["subject"]]["absent"] + int(not obj["did_attend"])
            day_present += int(obj["did_attend"])
            day_absent += int(not obj["did_attend"])
            sessions.append(obj)

    attendance_row["classes"] = sessions
    attendance_row["present"] = day_present
    attendance_row["absent"] = day_absent
    attendance_row["total"] = day_present + day_absent
    return attendance_row



from asyncio.exceptions import TimeoutError

from vasvscrapper.network_requests import get_results_html
from vasvscrapper.results.utilities import result_scrapper
from vasvscrapper.status_codes.results import RESULT_SCRAPPER_FAILED, RESULT_SUCCESS, RESULT_VCE_CRASHED


async def get_result(link: str) -> dict:
    """
    Extracts the results information and return a dictionary.

    PARAMETERS
    ----------
    link    : str
        Link of the result page

    RETURNS
    -------
    obj : dict
        Extracted information for a given page
    """

    results = {}
    try:
        status, result_details_soup = await get_results_html(link, 120)
        if status >= 400:
            results["status"] = RESULT_VCE_CRASHED

        else:
            subject_marks, semester_marks = await result_scrapper(result_details_soup)
            results["subjects"] = subject_marks
            results["total"] = semester_marks
            results["status"] = RESULT_SUCCESS

        return results

    except TimeoutError:
        results["status"] = RESULT_VCE_CRASHED
        return results

    except Exception:
        results["status"] = RESULT_SCRAPPER_FAILED
        return results



"""
News status codes representing failures and success states in scrapping news.
"""

NEWS_VCE_CRASHED = "NEWS__VCE_CRASHED"
NEWS_SCRAPPER_FAILED = "NEWS__SCRAPPER_FAILED"
NEWS_SUCCESS = "NEWS__SUCCESS"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['vasvscrapper',
 'vasvscrapper.attendance',
 'vasvscrapper.attendance.utilities',
 'vasvscrapper.enums',
 'vasvscrapper.login',
 'vasvscrapper.login.utilities',
 'vasvscrapper.network_requests',
 'vasvscrapper.news',
 'vasvscrapper.news.utilities',
 'vasvscrapper.results',
 'vasvscrapper.results.utilities',
 'vasvscrapper.status_codes']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.7.4,<4.0.0', 'bs4>=0.0.1,<0.0.2', 'lxml>=4.6.3,<5.0.0']

setup_kwargs = {
    'name': 'vasvscrapper',
    'version': '0.0.2a2',
    'description': '',
    'long_description': None,
    'author': 'P. Soumith Reddy',
    'author_email': 'soumithreddypodduturi@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import aiohttp
from bs4 import BeautifulSoup
from yarl import URL


async def get_login_html(roll_number, password, timeout=20) -> tuple[int, BeautifulSoup, URL]:
    """
    Logs in the student and returns status
    and BeautifulSoup object of the request.

    PARAMETERS
    ----------
    roll_number    : str
        Roll number of the student in vce.
    password       : str
        Password of the student in vce portal.
    timeout        : int
        Timeout of the page before it terminates fetching (default is 20 seconds).

    RETURNS
    -------
    obj: tuple
        HTTP status of a given page, BeautifulSoup object of the response, url of the response.
    """

    params = {"LoginID": roll_number, "Password": password}
    async with aiohttp.ClientSession() as session:
        async with session.post("https://vce.ac.in/", data=params, ssl=False, timeout=timeout) as resp:
            return resp.status, BeautifulSoup(await resp.text(), "lxml"), resp.url


async def get_news_html(page_number) -> tuple[int, BeautifulSoup]:
    """
    Fetches the News webpage and returns status
    and BeautifulSoup object of the request.

    PARAMETERS
    ----------
    page_number    : int
        Page number in vce website.

    RETURNS
    -------
    obj: tuple
        HTTP status of a given page, BeautifulSoup object of the response.
    """

    payload = {"page": page_number}
    async with aiohttp.ClientSession() as session:
        async with session.get("https://vce.ac.in/News_Events", params=payload, ssl=False) as resp:
            return resp.status, BeautifulSoup(await resp.text(), "lxml")


async def get_attendance_html(link, timeout=120):
    """
    Fetches the Attendance webpage and returns status
    and BeautifulSoup object of the request.

    PARAMETERS
    ----------
    link    : str
        Link of attendance page in vce website.
    timeout: int
        Timeout of the page before it terminates fetching (default is 120 seconds).

    RETURNS
    -------
    obj: tuple
        HTTP status of a given page, BeautifulSoup object of the response.
    """

    async with aiohttp.ClientSession() as session:
        async with session.get(link, ssl=False, timeout=timeout) as resp:
            return resp.status, BeautifulSoup(await resp.text(), "html.parser")


async def get_results_html(link, timeout=120):
    """
    Fetches the Results webpage and returns status
    and BeautifulSoup object of the request.

    PARAMETERS
    ----------
    link    : str
        Link of attendance page in vce website.
    timeout: int
        Timeout of the page before it terminates fetching (default is 120 seconds).

    RETURNS
    -------
    obj: tuple
        HTTP status of a given page, BeautifulSoup object of the response.
    """

    async with aiohttp.ClientSession() as session:
        async with session.get(link, ssl=False, timeout=timeout) as resp:
            return resp.status, BeautifulSoup(await resp.text(), "html.parser")



"""
Result status codes representing failures and success states in scrapping results.
"""

RESULT_VCE_CRASHED = "RESULT__VCE_CRASHED"
RESULT_SCRAPPER_FAILED = "RESULT__SCRAPPER_FAILED"
RESULT_SUCCESS = "RESULT__SUCCESS"



import asyncio


async def semester_marks_scrapper(marks_rows, headings, headings_main):
    total = {}
    for heading in headings:
        total[heading] = 0

    for heading in headings_main:
        total[heading] = 0

    marks_row = marks_rows[-2].find_all("td")
    p = 0
    for j in marks_row[1:-3]:
        if j.text == "-":
            inner_entry = 0
        elif j.text == "Ab" or j.text == "A":
            inner_entry = -1
        else:
            inner_entry = j.text
        total[f"{headings[p]}"] = float(inner_entry)
        p += 1
    total["ext_sub_credits"] = int(marks_row[-2].text)
    total["ext_grade_pts"] = int(marks_row[-1].text)
    marks_row = marks_rows[-1].find_all("td")
    p = 0
    for j in marks_row[1:-1]:
        if j.text == "-":
            inner_entry = 0
        elif j.text == "Ab" or j.text == "A":
            inner_entry = -1
        else:
            inner_entry = j.text
        total[f"{headings_main[p]}"] = float(inner_entry)
        p += 1

    if marks_row[-1].text.split()[-1] != "-":
        total["sgpa"] = float(marks_row[-1].text.split()[-1])
    else:
        total["sgpa"] = None

    return total


async def subject_marks_scrapper(marks_rows, headings):
    subjects = {}
    for mark in marks_rows[3:-2]:
        marks_row = mark.find_all("td")
        sub = {}
        for heading in headings:
            sub[heading] = 0
        p = 0
        for j in marks_row[2:-3]:
            if j.text == "-":
                inner_entry = 0
            elif j.text == "Ab" or j.text == "A":
                inner_entry = -1
            else:
                inner_entry = j.text
            sub[f"{headings[p]}"] = float(inner_entry)
            p += 1
        if marks_row[-3].text != "-":
            sub["ext_grade"] = marks_row[-3].text
        else:
            sub["ext_grade"] = "NA"
        sub["ext_sub_credits"] = int(marks_row[-2].text)
        sub["ext_grade_pts"] = int(marks_row[-1].text)
        subjects[marks_row[1].text.split("(")[0]] = sub

    return subjects


async def result_scrapper(result_details_soup):
    result_inner = result_details_soup.find(id="pnlContents")
    subject_marks_soup = result_inner.find_all(class_="tableclass")[1]
    headings = []
    headings_main = []
    marks_rows = subject_marks_soup.find_all("tr")
    for marks_row in marks_rows[1].find_all("td")[2:-2]:
        for i in range(1, 3):
            if marks_row.text.split()[0] == f"Int{i}":
                headings.append(f"int{i}_max")
                headings.append(f"int{i}")
                headings_main.append(f"int{i}_per")

        for i in range(1, 4):
            if marks_row.text.split()[0] == f"Asst{i}":
                headings.append(f"assn{i}_max")
                headings.append(f"assn{i}")
                headings_main.append(f"assn{i}_per")

            if marks_row.text.split()[0] == f"Quiz{i}":
                headings.append(f"quiz{i}_max")
                headings.append(f"quiz{i}")
                headings_main.append(f"quiz{i}_per")
    headings.append("sess_max")
    headings.append("sess")
    headings_main.append("sess_per")
    result = await asyncio.gather(
        subject_marks_scrapper(marks_rows=marks_rows, headings=headings),
        semester_marks_scrapper(marks_rows=marks_rows, headings=headings, headings_main=headings_main),
    )
    return result
